@@ -139,8 +139,6 @@ char* ReadExifDate(char* File, int* IncludesGPS)
 		*IncludesGPS = 1;
 	}
 
-	GPSData = ExifRead["Exif.GPSInfo.GPSMapDatum"];
-
 	// Now return, passing a pointer to the date string.
 	return Copy; // Its up to the caller to free this.
 };
@@ -191,7 +189,7 @@ char* ReadExifData(char* File, double* Lat, double* Long, double* Elev, int* Inc
 	strcpy(Copy, Value.c_str());
 	
 	// Check if we have GPS tags.
-	Exiv2::Exifdatum& GPSData = ExifRead["Exif.GPSInfo.GPSVersionID"];
+	Exiv2::Exifdatum GPSData = ExifRead["Exif.GPSInfo.GPSVersionID"];
 
 	Value = GPSData.toString();
 
@@ -219,44 +217,58 @@ char* ReadExifData(char* File, double* Lat, double* Long, double* Elev, int* Inc
 		//     -- / 3600 = result
 		//      v   
 		// Each part is added to the final number.
+		Exiv2::Rational RatNum;
+
 		GPSData = ExifRead["Exif.GPSInfo.GPSLatitude"];
-		Exiv2::Rational RatNum = GPSData.toRational(0);
-		*Lat = (double)RatNum.first / (double)RatNum.second;
-		RatNum = GPSData.toRational(1);
-		*Lat = *Lat + (((double)RatNum.first / (double)RatNum.second) / 60);
-		RatNum = GPSData.toRational(2);
-		*Lat = *Lat + (((double)RatNum.first / (double)RatNum.second) / 3600);
-		
-		GPSData = ExifRead["Exif.GPSInfo.GPSLatitudeRef"];
-		if (strcmp(GPSData.toString().c_str(), "S") == 0)
-		{
-			// Negate the value - Western Hemisphere.
-			*Lat = -*Lat;
+		if (GPSData.count() < 3)
+			*Lat = nan("invalid");
+		else {
+			RatNum = GPSData.toRational(0);
+			*Lat = (double)RatNum.first / (double)RatNum.second;
+			RatNum = GPSData.toRational(1);
+			*Lat = *Lat + (((double)RatNum.first / (double)RatNum.second) / 60);
+			RatNum = GPSData.toRational(2);
+			*Lat = *Lat + (((double)RatNum.first / (double)RatNum.second) / 3600);
+
+			GPSData = ExifRead["Exif.GPSInfo.GPSLatitudeRef"];
+			if (strcmp(GPSData.toString().c_str(), "S") == 0)
+			{
+				// Negate the value - Western Hemisphere.
+				*Lat = -*Lat;
+			}
 		}
 		
 		GPSData = ExifRead["Exif.GPSInfo.GPSLongitude"];
-		RatNum = GPSData.toRational(0);
-		*Long = (double)RatNum.first / (double)RatNum.second;
-		RatNum = GPSData.toRational(1);
-		*Long = *Long + (((double)RatNum.first / (double)RatNum.second) / 60);
-		RatNum = GPSData.toRational(2);
-		*Long = *Long + (((double)RatNum.first / (double)RatNum.second) / 3600);
-		
-		GPSData = ExifRead["Exif.GPSInfo.GPSLongitudeRef"];
-		if (strcmp(GPSData.toString().c_str(), "W") == 0)
-		{
-			// Negate the value - Western Hemisphere.
-			*Long = -*Long;
+		if (GPSData.count() < 3)
+			*Long = nan("invalid");
+		else {
+			RatNum = GPSData.toRational(0);
+			*Long = (double)RatNum.first / (double)RatNum.second;
+			RatNum = GPSData.toRational(1);
+			*Long = *Long + (((double)RatNum.first / (double)RatNum.second) / 60);
+			RatNum = GPSData.toRational(2);
+			*Long = *Long + (((double)RatNum.first / (double)RatNum.second) / 3600);
+
+			GPSData = ExifRead["Exif.GPSInfo.GPSLongitudeRef"];
+			if (strcmp(GPSData.toString().c_str(), "W") == 0)
+			{
+				// Negate the value - Western Hemisphere.
+				*Long = -*Long;
+			}
 		}
 
 		// Finally, read elevation out. This one is simple.
 		GPSData = ExifRead["Exif.GPSInfo.GPSAltitude"];
-		RatNum = GPSData.toRational(0);
-		*Elev = (double)RatNum.first / (double)RatNum.second;
+		if (GPSData.count() < 1)
+			*Elev = nan("invalid");
+		else {
+			RatNum = GPSData.toRational(0);
+			*Elev = (double)RatNum.first / (double)RatNum.second;
+		}
 
 		// Is the altitude below sea level? If so, negate the value.
 		GPSData = ExifRead["Exif.GPSInfo.GPSAltitudeRef"];
-		if (strcmp(GPSData.toString().c_str(), "1") == 0)
+		if (GPSData.count() >= 1 && GPSData.toLong() == 1)
 		{
 			// Negate the elevation.
 			*Elev = -*Elev;
@@ -328,16 +340,28 @@ char* ReadGPSTimestamp(char* File, char* DateStamp, char* TimeStamp, int* Includ
 	} else {
 		// Seems to include GPS data...
 		*IncludesGPS = 1;
-		
+
+		Exiv2::Rational RatNum1;
+		Exiv2::Rational RatNum2;
+		Exiv2::Rational RatNum3;
+
 		// Read out the Time and Date stamp, for correction.
 		GPSData = ExifRead["Exif.GPSInfo.GPSTimeStamp"];
-		Exiv2::Rational RatNum1 = GPSData.toRational(0);
-		Exiv2::Rational RatNum2 = GPSData.toRational(1);
-		Exiv2::Rational RatNum3 = GPSData.toRational(2);
+		if (GPSData.count() < 3) {
+			*IncludesGPS = 0;
+			return Copy;
+		}
+		RatNum1 = GPSData.toRational(0);
+		RatNum2 = GPSData.toRational(1);
+		RatNum3 = GPSData.toRational(2);
 		snprintf(TimeStamp, 12, "%02d:%02d:%02d",
 				RatNum1.first, RatNum2.first, RatNum3.first);
 		
 		GPSData = ExifRead["Exif.GPSInfo.GPSDateStamp"];
+		if (GPSData.count() < 3) {
+			*IncludesGPS = 0;
+			return Copy;
+		}
 		RatNum1 = GPSData.toRational(0);
 		RatNum2 = GPSData.toRational(1);
 		RatNum3 = GPSData.toRational(2);
@@ -469,7 +493,7 @@ int WriteGPSData(char* File, struct GPSPoint* Point, char* Datum, int NoChangeMt
 	ExifToWrite.add(Exiv2::ExifKey("Exif.GPSInfo.GPSAltitudeRef"), Value.get());
 	// And the actual altitude.
 	Value = Exiv2::Value::create(Exiv2::unsignedRational);
-	ConvertToRational(Point->Elev, &Nom, &Denom, 4);
+	ConvertToRational(fabs(Point->Elev), &Nom, &Denom, 4);
 	snprintf(ScratchBuf, 100, "%ld/%ld", Nom, Denom);
 
 	/* printf("Altitude: %f -> %s\n", Point->Elev, ScratchBuf); */
