@@ -33,7 +33,9 @@ extern "C" {
 	
 char* ReadExifDate(char* File, int* IncludesGPS);
 char* ReadExifData(char* File, double* Lat, double* Long, double* Elevation, int* IncludesGPS);
-int WriteGPSData(char* File, struct GPSPoint* Point, char* Datum, int NoChangeMtime);
+char* ReadGPSTimestamp(char* File, char* DateStamp, char* TimeStamp, int* IncludesGPS);
+int WriteGPSData(char* File, struct GPSPoint* Point, char* Datum, int NoChangeMtime, int DegMinSecs);
+int WriteFixedDatestamp(char* File, time_t TimeStamp);
 int RemoveGPSExif(char* File, int NoChangeMtime);
 
 #ifdef __cplusplus
